function aufruf(problem)

% aufruf(problem legt die Variablen und Parameter fuer das 
% entsprechende Problem fest und ruft das Iterationsverfahren 
% fuer das entsprechende Problem auf.

% Die Variable problem ist entweder eines der Probleme
% 'wm'    =   Wachstumsmodell                     (Beispiel 1)
% 'oek'   =   Oekologieproblem                    (Beispiel 2)
% 'im'    =   Investitionsmodell                  (Beispiel 3)
% 'kred'  =   Wachstumsmodell mit Kreditaufnahme  (Beispiel 4)
%
% oder ein neues Problem 'neu', das durch die Funktionen fh_neu 
% und g_neu definiert ist.


tic;                            % Start der Messung der Rechenzeit

switch problem
case 'wm'
    fh          =   @fh_wm;             
    g           =   @g_wm;              
    % Aufruf der das Problem bestimmenden Funktionen;
    % als M-file unter fh_. bzw g_. abgespeichert.
    
    startknoten =   17;                 
    % Anzahl der Knoten des Ausgangsgitters
    x_1         =   0.1;                
    % linke Grenze des zu betrachtenden Bereichs [x_1,x_n]
    x_n         =   10;                 
    % rechte Grenze des zu betrachtenden Bereichs [x_1,x_n]
    x           =   x_1:((x_n-x_1)/(startknoten-1)):x_n;    
    % Speicherung aller Knoten des Ausgangsgitters im Vektor x 

    kontrollen  =   500;                
    % Anzahl der Kontrollwerte aus der Diskretisierung von U
    u_1         =   0.1;                
    % linke Grenze des Kontrollwertebereichs U
    u_m         =   11;                 
    % rechte Grenze des Kontrollwertebereichs U
    u           =   u_1:((u_m-u_1)/(kontrollen-1)):u_m;     
    % Speicherung aller Kontrollwerte im Vektor u

    h           =   1;                  
    % Schrittweite der zeitlichen Diskretisierung
    delta       =   0.05;               
    % Diskontrate
    
    zeta        =   1e-5;               
    % Abbruchparamter; bestimmt, ab wann die Approximation auf dem 
    % betrachteten Gitter genau genug ist
    
    theta       =   0.1;                
    % Verfeinerungsparamter; bestimmt, welche Intervalle bei der
    % Gitteradaption verfeinert werden
                                        
    grenze_kn   =   300;                
    % maximale Knotenzahl; hat das betrachtete Gitter mehr Knoten,
    % wird nicht mehr verfeinert
   
    x_vgl       =   x_1:((x_n-x_1)/(1000-1)):x_n;                  
    v_vgl       =   exakt_wm(x_vgl);        
    % Die bekannten exakten Werte dienen als Vergleichswerte.
                                            
case 'oek'      % Variablen analog zu oben
    fh          =   @fh_oek;            
    g           =   @g_oek;             
    
    startknoten =   50;                 
    x_1         =   0;                  
    x_n         =   3;                  
    x           =   x_1:((x_n-x_1)/(startknoten-1)):x_n;     

    kontrollen  =   50;                 
    u_1         =   0;                  
    u_m         =   0.4;                
    u           =   u_1:((u_m-u_1)/(kontrollen-1)):u_m;     

    h           =   0.05;               
    delta       =   0.1;                
    
    zeta        =   1e-5;               
                                       
    theta       =   0.1;                
                                        
    grenze_kn   =   225;                
    
    [x_vgl, v_vgl]=loesung_linear('oek'); 
    % Die Loesungen, die mit kontrolliertem Gauss-Seidel-Verfahren
    % errechnet wurden, werden eingelesen. Sie dienen hier als 
    % Vergleichswerte.
    
case 'im'           % Variablen analog zu oben
    fh          =   @fh_im;             
    g           =   @g_im;              
    
    startknoten =   131;                
    x_1         =   0;                  
    x_n         =   0.7;                
    x           =   x_1:((x_n-x_1)/(startknoten-1)):x_n;    

    kontrollen  =   50;                 
    u_1         =   0;                  
    u_m         =   1;                  
    u           =   u_1:((u_m-u_1)/(kontrollen-1)):u_m;     

    h           =   0.05;              
    delta       =   0.2;                
    
    zeta        =   1e-5;               
    
    theta       =   0.3;                
                                        
    grenze_kn   =   307;                
     
    [x_vgl, v_vgl]=loesung_linear('im');
    
case 'kred'
    fh          =   @fh_kred;           
    g           =   @g_kred;            
    
    startknoten =   39;                 
    x_1         =   0.0001;             
    x_n         =   2.0001;             
    x           =   x_1:((x_n-x_1)/(startknoten-1)):x_n;    

    kontrollen  =   50;                
    u_1         =   0;                  
    u_m         =   0.25;               
    u           =   u_1:((u_m-u_1)/(kontrollen-1)):u_m;     

    h           =   0.05;               
    delta       =   0.1;               
    
    zeta        =   1e-5;               
    
    theta       =   0.5;                
                                        
    grenze_kn   =   116;                
                                        
    [x_vgl, v_vgl]=loesung_linear('kred'); 
    
case 'neu'  % Hier koennen die Daten fuer ein weiteres Problem 
            % abgespeichert werden.
    fh          =   @fh_neu;             
    g           =   @g_neu;             
    
    startknoten =   1;                  
    x_1         =   0;                 
    x_n         =   1;                  
    x           =   x_1:((x_n-x_1)/(startknoten-1)):x_n;    

    kontrollen  =   1;                  
    u_1         =   0;                  
    u_m         =   1;                 
    u           =   u_1:((u_m-u_1)/(kontrollen-1)):u_m;     

    h           =   0.1;                
    delta       =   0.1;                
    
    zeta        =   1e-5;               
    
    theta       =   0.1;                
                                        
    grenze_kn   =   11;                 
                                        
    [x_vgl, v_vgl]=loesung_linear('neu'); 
    
otherwise     
    fprintf('Unbekanntes Problem!');
    break;
end

% Aufruf des Iterationsbverfahrens mit den festgelegten
% Parametern und Funktionen

beta    =   1-delta*h;                  % Diskontfaktor wird berechnet

verf_mit_spline(fh,g,x,u,h,delta,beta,zeta,theta,grenze_kn,x_vgl,v_vgl,problem);

toc                                     % Ausgabe der gesamten Rechenzeit