function verf_mit_spline(fh,g,x,u,h,delta,beta,zeta,theta,grenze_kn,x_vgl,v_vgl,problem);

% Die Funktion 'verfahren_mit_spline.m' berechnet 
% Optimalwertfunktion eines eindimensionalen optimalen 
% Steuerungsproblems mit MATLAB-Spline-Interpolationsroutine
% auf adaptiven Gittern.
%
% In jeder Iteration wird die maximale Differenz zwischen der
% aktuellen und der vorherigen Approximation sowie die Anzahl
% der momentanen Knoten ausgegeben. Ausserdem wird nach jeder 
% Hauptiteration (d.h. auf dem aktuellen Gitter ist die optimale 
% Wertefunktion bestimmt) eine obere Grenze fuer den Fehler auf 
% dem gesamten betrachteten Bereich [x_1,x_n] angezeigt.

z_iter  =   0;                          
% Iterationszaehler wird auf Null gesetzt.                      

% FESTLEGUNG DER VOM BETRACHTETEN PROBLEM UNABHAENGIGEN PARAMETER

grenze_iter     =   3000;               
% Sind grenze_iter Iterationen durchlaufen, wird abgebrochen
% (noetig bei periodischer Entwicklung der Optimalwertfunktion)

plotintervall   =   200;                
% Gibt an, nach wie vielen Iterationen die momentane 
% Optimalwertfunktion jeweils geplottet werden soll.

% VORBESETZUNG DER VEKTOREN

ff   = zeros(length(x),length(u));
gg   = zeros(length(x),length(u));
uopt = ones(1,length(x));
v    = zeros(length(x),1);

abbruch = 0;                            
% Wenn Variable abbruch=1, wird der Algorithmus beendet


% DURCHLAUFE SCHLEIFE SO LANGE, BIS EIN ABBRUCHKRITERIUM ERFUELLT IST

while(abbruch==0)
    for i=1:length(x)
        for k=1:length(u)
            ff(i,k) = feval(fh,x(i),u(k),h);                
            gg(i,k) = feval(g,x(i),u(k));                   
        end
    end
% Berechnung von fh an allen Knotenpunkten fuer alle Kontrollwerte
% Berechnung von g an allen Knotenpunkten fuer alle Kontrollwerte
    
    zetamax = 2*zeta;
      
    while(zetamax>zeta)                                     
    % Durchlaufe Schleife so lange, bis der maximale Schritt zetamax
    % in einer Iteration klein genug ist, d.h. bis                            
    % das Ergebnis auf dem betrachteten Gitter genau genug ist                                                       
       
       z_iter=z_iter+1;                                     
       % Erhoehe Iterationszaehler  
       
       ss=spline(x,v);                                      
       % Berechnung des Splines zu den Werten v_i an den Knoten x_i
       
       v2=v;                                                
       % Speicherung zur Bestimmung der Werte zeta
                             
           for i=1:length(x)
           
              for k=1:length(u)
                  if ff(i,k)>=x(1) & ff(i,k)<=x(length(x))              
                     % Es werden nur die Knoten x betrachtet, bei
                     % denen fh(x,u) noch im Bereich [x_1,x_n] liegt
                      
                     ss2(k)=ppval(ss,ff(i,k));                         
                     % Auswertung der Splines an den Werten von ff 
                     
                     t(k)=h*gg(i,k)+beta*ss2(k);                       
                     % Berechnung der Vergleichswerte, aus denen
                     % anschliessend das Maximum und so der neue 
                     % Wert v_i am Knoten x_i berechnet wird. 
                     % Sie werden abgespeichert im Vektor t.
                     
                  else                                          
                      t(k)=-1e+5;                                       
                      % Fuer alle u_k, fuer die der Wert fh(x_i,u_k) 
                      % nicht im Bereich [x_1,x_n] liegt, wird der 
                      % Vergleichswert so  besetzt,dass er bei der 
                      % Maximierung keine Rolle spielt.
                  end
              end
              
              v(i)=max(t);                                   
              % Bestimmung des neuen Wertes v_i am Knoten x_i 
              % durch Maximierung
    
              for j=1:length(t)                              
                  if t(j)==max(t)                            
                     uopt(i)=u(j);
                  end
              end
              % Speicherung der optimalen Kontrollwerte zu 
              % jedem Knoten x_i im Vektor uopt.
              clear t;
              clear ss2;
           end;
       
       zeta2=v-v2;                                             
       % Berechnung der "Schritte" zeta_i in der letzten Iteration, 
       % abgespeichert im Vektor zeta2.
       
       zetamax=max(abs(zeta2));                                
       % Bestimmung des Betrags-Maximums von zeta2.
       % Es zeigt an, ob die Approximation auf dem betrachteten Gitter
       % genau genug ist.
       
       fprintf('\nmax. Schritt: %f      %d      %d',zetamax,length(x),z_iter);     
       % Ausgabe maximaler Schritt, Knotenzahl, Iterationsanzahl

       maxschritt(z_iter)=zetamax;                             
       % Abspeichern des Wertes zetamax im Vektor maxschritt 
       % zur spaeteren Veranschaulichung
       % der Entwicklung im Laufe der Berechnungen 
        
       if z_iter>=grenze_iter                                   
           grenze_kn=1;                                    
           break;                                              
       end
       % Sind grenze_iter Iterationen bereits durchlaufen wird der 
       % Algorithmus beendet. Dazu wird die Begrenzung der Knotenzahl 
       % grenze_knoten auf 1 gesetzt und die while-Schleife verlassen.
       
       if problem(1:2)=='wm'                                                                 
            if mod(z_iter,plotintervall)==0                                             
                xhilf2 = x(1):(x(length(x))-x(1))/3000:x(length(x));                    
                for i=1:length(xhilf2)                                                  
                        vhilf2(i)=ppval(ss,xhilf2(i));                                  
                end                                                                     
                figure;                                                                  
                plot(xhilf2,vhilf2,x,v,'o',x_vgl, v_vgl);                              
                title('Entwicklung der Approximation');                                 
            end                                                                            
       else if mod(z_iter,plotintervall)==0                                              
                xhilf2 = x(1):(x(length(x))-x(1))/3000:x(length(x));                    
                for i=1:length(xhilf2)                                                  
                        vhilf2(i)=ppval(ss,xhilf2(i));                                                                
                end                                                                        
                figure;                                                                 
                plot(xhilf2,vhilf2,x,v,'o',x_vgl, v_vgl,'r'); 
                title('Entwicklung der Approximation');
            end
       end                                                                               
       % Nach jeweils "plotintervall" Iterationen wird 
       % die momentane Approximation der optimalen Wertefunktion
       % geplottet. Um die Form des Splines deutlicher darzustellen,
       % werden 3000 Hilfspunkte erzeugt und der Spline an diesen
       % ausgewertet. Zusaetzlich wird zum Vergleich die mit linearer
       % Interpolation ermittelte Optimalwertfunktion ausgegeben.
       % Ist die exakte optimale Wertefunktion bekannt (hier bei Beispiel 1)
       % wird die exakte Funktion als Vergleichsfunktion markiert.
       % Der Wert v_i an den Knoten x_i wird zusaetzlich
       % als Kreis eingezeichnet.
       % Durch diese regelmaessige Ausgabe kann die Entwicklung der
       % Approximationen nachvollzogen werden. 
       
   end         % Ende der inneren While-Schleife 
   
    figure;                                                                                        
    plot(x,v,x_vgl, v_vgl,'r');                             
    title('Approximation vor Gitteradaption');
    % Die auf dem betrachteten Gitter ermittelte Optimalwertfunktion 
    % wird ausgegeben. Wieder zum Vergleich zusammen mit der 
    % mit lin. Interpolation ermittelten Approximation.
   
    abbruch=1;                                               
    % Den Abbruch bestimmende Variable wird auf eins (=Abbruch) gesetzt.
    
    if problem(1:2)=='wm'
        % Ist die exakte Optimalwertfunktion bekannt (in dieser Arbeit 
        % bei Beispiel 1), wird die Approximation mit dieser verglichen.
        % Das geschieht an 1000 Hilfspunkten des Vektors x_vgl, der
        % durch die Aufruffunktion mit uebergeben wird.
                                                                                                                           
        ss=spline(x,v);                                      
        % Bestimmung des Splines mit den neuen Werten v_i.
        
        z2=1;            % Hilfszaehler
           
        for i=1:length(x_vgl)                               
            vhilf3(i)=ppval(ss,x_vgl(i));                   
        end
        % Auswertung des Splines an den 1000 Hilfspunkten 
        % des Vektors x_vgl (aus 'aufruf.m').
        % Die Werte werden abgespeichert im Vektor vhilf3.
        
        maxfehl(z2)=max(abs(vhilf3-v_vgl));       
        % Der maximale Fehler wird bestimmt durch Vergleich
        % der Werte der Approximation mit denen  
        % der exakten Optimalwertfunktion an den Hilfspunkten.                                                      
        % Diese sind im Vektor v_vgl mit uebergeben worden.
        
        fprintf('     max. Fehler=    %d',maxfehl(z2));     
        % Ausgabe des maximalen Fehlers der aktuellen Approximation
        
        z2=z2+1;
    end
    
% GITTERVERAENDERUNG
     
    ss=spline(x,v);                                            
    % Berechnung des Splines zu den neuen Werten v_i an den Knoten x_i

    for i=1:length(x)-1                              
        xx(i)=((x(i+1)+x(i))/2);                               
        % Alle Zellen werden halbiert und die so gewonnenen Testpunkte
        % im Vektor xx abgespeichert. Durch die Werte an den Testpunkten
        % wird der loakle Fehler abgeschaetzt.
        
        vv(i)=ppval(ss,xx(i));                                 
        % Berechnung des Wertes an den Testpunkten durch Auswertung des 
        % zu x und v gehoerigen Splines an den Testpunkten
    end
    
    for i=1:length(xx)
        for k=1:length(u)
            ff2(i,k) = feval(fh,xx(i),u(k),h);                 
            % Berechnung von fh an den Testpunkten
            
            gg2(i,k) = feval(g,xx(i),u(k));                    
            % Berechnung von g an den Testpunkten
        end;
    end;
       
    for i=1:length(xx)
    
        for k=1:length(u)
                  if ff2(i,k)>=x(1) & ff2(i,k)<=x(length(x))              
                      t2(k)=(h*gg2(i,k)+beta.*ppval(ss,ff2(i,k)));        
            % Fuer jeden Testpunkt wird analog zu oben eine Iteration 
            % durchgefuehrt. Die Vergleichswerte werden bestimmt und im  
            % Vektor t2 abgespeichert. 
            
                  else                                                    
                      t2(k)=-1e+5;                                        
                  end
            % Die Werte fuer diejenigen Kontrollen u_k,
            % fuer die fh(xx_i, u_k) nicht in [x_1,x_n] liegt,
            % werden "ausgeschlossen".
        end
    
        vtest(i)=max(t2);                                           
        % Bestimmung des neuen Wertes am Punkt xx_i durch Maximierung.  
        % Die neuen Werte an den Testpunkten werden
        % im Vektor vtest abgespeichert.
        
        clear t2;                                                
    end                                                         
    
    for i=1:length(xx)                                          
         eta(i)=abs(vtest(i)-vv(i));                                        
    end                                                        
    % Bestimmung der betragsmaessigen Groesse     
    % eines Schrittes in allen Testpunkten, durch Vergleich 
    % mit dem alten Wert. Abgespeichert im Vektor eta.
    % Durch diesen Wert ist der Fehler im Intervall begrenzt.

    maxeta=max(eta);   
    % Bestimmung des Residuums durch Maximum-Bestimmung des Vektors eta

    fprintf('        eta: %f   ',maxeta);                                       
    % Ausgabe des Residdums zum betrachteten Gitter
    fprintf('        Gesamtfehlerschranke: %f    ',maxeta*(1/(1-beta)));        
    % Ausgabe der oberen Schranke fuer den Gesamtfehler gemaess Satz 5.4
    
    figure;                                                     
    plot(xx,eta,'o');                                           
    title('lokaler Fehler in den einzelnen Intervallen');
    % Ausgabe der Schritte und somit des Fehlers am jeweiligen Knoten.

    % NEUE KNOTENPUNKTE:

    if(length(x)<grenze_kn)                                     
    % Neues Gitter wird nur gebildet, wenn die maximale Knotenzahl 
    % noch nicht erreicht ist.
                                                                
        z3=1;       % Hilfszaehler 
        
        for i=1:length(xx)                                      
             if eta(i)>theta*maxeta                              
               xneu(z3)=xx(i);                                  
               vneu(z3)=vv(i);                                  
               z3=z3+1;
            end
        end
  % Die Knoten, an denen der Fehler vergleichsweise gross ist, werden
  % im Vektor xneu abgespeichert. Die zugehoerigen Werte im Vektor vneu. 
  % Die Variable 0 < theta < 1 wird in aufruf.m festgesetzt.

        xhilf=x;                                                
        vhilf=v;    
  % Hilfsvektoren zur Erstellung des Gitters fuer die 
  % naechste Hauptiteration
  
  
        [x,v]=vektsort(xhilf,vhilf,xneu,vneu);                   
        % Erstellung der Vektoren fuer die naechste Hauptiteration durch
        % Hilfsfunktion vektsort.
        % vektsort gibt die alten Knoten aus x und die neuen Knoten aus
        % xneu sortiert im neuen Vektor x aus. Die zugehoerigen Werte 
        % stehen an den entsprechenden Stellen im neuen Vektor v.
        
        clear xneu;
        clear vneu;
     
        if maxeta>zeta/(1/(1-beta))                              
            abbruch = 0;                                         
            clear uopt;                                          
        end
        % Wenn der maximale Fehler nicht klein genug ist, wird die
        % Abbruchvariable auf Null gesetzt und man geht zur naechsten 
        % Hauptiteration. Es wird also die Approximation auf dem neuen 
        % Gitter mit den Knoten x_i bestimmt. 
        % Ansonsten wird der Algorithmus beendet. 
    end                                                          

end         % Ende der aeusseren While-Schleife

% VERANSCHAULICHUNG DER ERRECHNETEN ERGEBNISSE

intlaengen=diff(x);                                             
% Berechnung der Intervalllaengen

intlaengen(length(x))=intlaengen(length(x)-1);                   
% Laenge des Vektors intlaengen wird an x angeglichen

erstelle_graphen(x, v, uopt, intlaengen, x_vgl, v_vgl,maxschritt);          
% Aufruf des Unterprogramms erstelle_graphen.
% Die errechneten Ergebnisse werden als Plots ausgegeben.     

